@@ -10,6 +10,8 @@ from asgikit.errors.websocket import (
 from asgikit.headers import MutableHeaders
 from asgikit.http_connection import HttpConnection
 
+__all__ = ("WebSocket",)
+
 
 class WebSocket(HttpConnection):
     class State(Enum):
@@ -17,6 +19,8 @@ class WebSocket(HttpConnection):
         ACCEPTED = 2
         CLOSED = 3
 
+    __slots__ = ("subprotocols", "_state")
+
     def __init__(self, scope, receive, send):
         assert scope["type"] == "websocket"
         super().__init__(scope, receive, send)
@@ -31,7 +35,7 @@ class WebSocket(HttpConnection):
         if self._state != self.State.NEW:
             raise WebSocketStateError()
 
-        message = await self.asgi.receive()
+        message = await self.asgi_callbacks.receive()
         if message["type"] != "websocket.connect":
             raise WebSocketError()
 
@@ -43,7 +47,7 @@ class WebSocket(HttpConnection):
             else:
                 return ValueError("headers")
 
-        await self.asgi.send(
+        await self.asgi_callbacks.send(
             {
                 "type": "websocket.accept",
                 "subprotocol": subprotocol,
@@ -57,7 +61,7 @@ class WebSocket(HttpConnection):
         if self._state != self.State.ACCEPTED:
             raise WebSocketStateError()
 
-        message = await self.asgi.receive()
+        message = await self.asgi_callbacks.receive()
         if message["type"] == "websocket.disconnect":
             self._state = self.State.CLOSED
             raise WebSocketDisconnectError(message["code"])
@@ -68,7 +72,7 @@ class WebSocket(HttpConnection):
         if self._state != self.State.ACCEPTED:
             raise WebSocketStateError()
 
-        await self.asgi.send(
+        await self.asgi_callbacks.send(
             {
                 "type": "websocket.send",
                 "text": data,
@@ -79,7 +83,7 @@ class WebSocket(HttpConnection):
         if self._state != self.State.ACCEPTED:
             raise WebSocketStateError()
 
-        await self.asgi.send(
+        await self.asgi_callbacks.send(
             {
                 "type": "websocket.send",
                 "bytes": data,
@@ -97,5 +101,5 @@ class WebSocket(HttpConnection):
         if code:
             message["code"] = code
 
-        await self.asgi.send(message)
+        await self.asgi_callbacks.send(message)
         self._state = self.State.CLOSED
